/* 	Kurs programowania mikrokontrolerow Megawin
		Projekt 03 - podstawowa obsluga ADC */

#include "mg32f10x.h"
#include "hardware.h"


int main(void)
{
	
	int16_t cnt = 0;
	uint8_t pos = 0;
	
	/* liczba probek ktore beda usredniane */
	const uint8_t num_samples = 128;

	/* zmienna do przechowywania wyniku konwersji */
	uint16_t adc_tmp = 0;
	
	/* akumulator do usredniania probek */
	uint32_t volt_avg = 0;
	
	/* zmienna przechowujaca liczbe do wyswietlenia */
	uint16_t disp_num = 0;

	initPeripherals();
	
	LED_onoff(1, LED_OFF);
	LED_onoff(2, LED_OFF);
	
  while (1)
  {
		
		volt_avg = 0;
		
		/* zapelnianie bufora probek i sumowanie wynikow "w locie" */
		for(uint8_t i = 0; i < num_samples; i++){
		
			/* rozpoczecie pojedynczej konwersji ADC */
			ADC_SoftwareStartConvCmd(ENABLE);
			
			/* polling flagi konca konwersji */
			while(ADC_GetFlagStatus(ADC_FLAG_EOC) != SET){}
			
			/* przeliczanie wyniku na mV (Vref = Vcc = 3,3 V) */
			adc_tmp = ADC->DR;	
				
			/* nacisniecie SW2 powoduje zastosowanie "funkcji korygujacej" */
			if(!GPIO_ReadInputDataBit(SW2_port, SW2_pin))
				adc_tmp = ADC_GetADValue(adc_tmp);
				
			volt_avg += (adc_tmp * 3300) / 4096;			
		}
		
		/* zakonczenie obliczen (srednia arytmetyczna) */
		volt_avg /= num_samples;
		
		
		int8_t digits[4];

		/* Pozyskanie kolejnych pozycji dziesietnych */
		digits[0] = (disp_num % 10000) / 1000;
		digits[1] = (disp_num % 1000) / 100;
		digits[2] = (disp_num % 100) / 10;
		digits[3] = (disp_num % 10);

		/* Tymczasowe wylaczenie wyswietlacza */
		display_select_pos(255);
		
		/* Ustawienie cyfry do wyswietlenia na katodach */
		display_select_digit(digits[pos], (pos==0)?1:0);
		
		/* Wlaczenie wybranej pozycji */
		display_select_pos(pos);

		/* Obsluga licznika pozycji dziesietnych */
		pos++;
		if(pos > 3){
			pos = 0;
			disp_num = (uint16_t)volt_avg;
		}
	}

}


#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {
  }
}
#endif
